/*****************************************************************************/
/* Copyright (c) 2005, ZeitControl cardsystems GmbH                          */
/* All rights reserved.                                                      */
/*                                                                           */
/* Redistribution and use in source and binary forms, with or without        */
/* modification, are permitted provided that the following conditions are    */
/* met:                                                                      */
/*    * Redistributions of source code must retain the above copyright       */
/*      notice, this list of conditions and the following disclaimer.        */
/*    * Redistributions in binary form must reproduce the above copyright    */
/*      notice, this list of conditions and the following disclaimer in the  */
/*      documentation and/or other materials provided with the distribution. */
/*    * Neither the name of the company ZeitControl nor the names of its     */
/*      contributors may be used to endorse or promote products derived from */
/*      this software without specific prior written permission.             */
/*                                                                           */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       */
/* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED */
/* TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A           */
/* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER  */
/* OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,  */
/* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,       */
/* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR        */
/* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    */
/* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      */
/* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        */
/* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/*****************************************************************************/
/* Created on 01.04.2005
 * Project: BasicCard .Net Library
 * File description: 
 *   Component manager for MultiApplication BasicCard.
 * --------------------------------------------------------
 * $Header: /opt/cvsroot/net/BasicCardLib.net/BasicCard/BasicCards/BasicCardComponentManager.cs,v 1.2 2005/07/25 13:11:46 michael Exp $
 * $Revision: 1.2 $
 * $Date: 2005/07/25 13:11:46 $
 * --------------------------------------------------------
 * $Log: BasicCardComponentManager.cs,v $
 * Revision 1.2  2005/07/25 13:11:46  michael
 * *** empty log message ***
 *
 * Revision 1.1  2005/07/22 12:57:57  michael
 * Initial checkin
 *
 *---------------------------------------------------------
 */
using System;
using BasicCard.BasicCards;

namespace BasicCard.BasicCards.Components
{
	/// <summary>
	/// Class to access component functions from AbstractBasicCardService.
	/// </summary>
	[CLSCompliant(true)]
	public class BasicCardComponentManager
	{
		private AbstractBasicCardService service;

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="service">AbstractBasicCardService to use.</param>
		public BasicCardComponentManager(AbstractBasicCardService service)
		{
			this.service = service;
		}

		/// <summary>
		/// Find component CID from component name and type.
		/// </summary>
		/// <param name="componenttype">component type</param>
		/// <param name="componentname">component name (including path)</param>
		/// <returns>CID</returns>
		public int FindComponent(ComponentType componenttype, BasicCardString componentname)
		{
			return service.CmdFindComponent(componenttype, componentname);
		}

		/// <summary>
		/// Get component name.
		/// </summary>
		/// <param name="CID">CID of component</param>
		/// <returns>component name</returns>
		public BasicCardString ComponentName(int CID)
		{
			return service.CmdComponentName(CID);
		}

		/// <summary>
		/// Read componet attributes.
		/// </summary>
		/// <param name="CID">CID of component</param>
		/// <returns>component attributes</returns>
		public byte[] ReadAttr(int CID)
		{
			return service.CmdReadComponentAttr(CID);
		}

		/// <summary>
		/// Write componet attributes.
		/// </summary>
		/// <param name="CID">CID of component</param>
		/// <param name="attributes">component attributes</param>
		public void WriteAttr(int CID, byte[] attributes)
		{
			service.CmdWriteComponentAttr(CID, attributes);
		}

		/// <summary>
		/// Read component data.
		/// </summary>
		/// <param name="CID">CID of component</param>
		/// <returns>data</returns>
		public byte[] ReadData(int CID)
		{
			return service.CmdReadComponentData(CID);
		}

		/// <summary>
		/// Write component data.
		/// </summary>
		/// <param name="CID">CID of component</param>
		/// <param name="data">data</param>
		public void WriteData(int CID, byte[] data)
		{
			service.CmdWriteComponentData(CID, data);
		}

		/// <summary>
		/// Create a component within card.
		/// </summary>
		/// <param name="componenttype">componen type</param>
		/// <param name="componentname">component name/path</param>
		/// <param name="attributes">component attributes</param>
		/// <param name="data">component data</param>
		/// <returns>CID</returns>
		public int CreateComponent(
			ComponentType componenttype,
			BasicCardString componentname,
			byte[] attributes,
			byte[] data)
		{
			return service.CmdCreateComponent(
				componenttype,
				componentname,
				attributes,
				data);
		}

		/// <summary>
		/// Delete the specified component in card.
		/// </summary>
		/// <param name="cid">CID of component</param>
		public void DeleteComponent(int cid)
		{
			service.CmdDeleteComponent(cid);
		}
	}
}
