/*****************************************************************************/
/* Copyright (c) 2005, ZeitControl cardsystems GmbH                          */
/* All rights reserved.                                                      */
/*                                                                           */
/* Redistribution and use in source and binary forms, with or without        */
/* modification, are permitted provided that the following conditions are    */
/* met:                                                                      */
/*    * Redistributions of source code must retain the above copyright       */
/*      notice, this list of conditions and the following disclaimer.        */
/*    * Redistributions in binary form must reproduce the above copyright    */
/*      notice, this list of conditions and the following disclaimer in the  */
/*      documentation and/or other materials provided with the distribution. */
/*    * Neither the name of the company ZeitControl nor the names of its     */
/*      contributors may be used to endorse or promote products derived from */
/*      this software without specific prior written permission.             */
/*                                                                           */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       */
/* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED */
/* TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A           */
/* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER  */
/* OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,  */
/* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,       */
/* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR        */
/* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    */
/* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      */
/* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        */
/* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/*****************************************************************************/
/* Created on 01.04.2005
 * Project: BasicCard .Net Library
 * File description: 
 *   Public class for BasicCard card services.
 * --------------------------------------------------------
 * $Header: /opt/cvsroot/net/BasicCardLib.net/BasicCard/BasicCards/BasicCardService.cs,v 1.3 2009/03/18 12:50:41 michael Exp $
 * $Revision: 1.3 $
 * $Date: 2009/03/18 12:50:41 $
 * --------------------------------------------------------
 * $Log: BasicCardService.cs,v $
 * Revision 1.3  2009/03/18 12:50:41  michael
 * Release Candidate .Net 2.0
 *
 * Revision 1.2  2007/10/23 11:35:29  michael
 * Intermediate Version
 *
 * Revision 1.1  2005/07/22 12:57:57  michael
 * Initial checkin
 *
 *---------------------------------------------------------
 */
using System;
using BasicCard.Framework;
using BasicCard.GenericCards;
using BasicCard.BasicCards.FileSys;

namespace BasicCard.BasicCards
{
	/// <summary>
	/// Public implementation from AbstractBasicCardService, exporting protected
	/// functions as public.
	/// <seealso cref="AbstractBasicCardService"/>
	/// </summary>
	public sealed class BasicCardService : AbstractBasicCardService
	{
		/// <overloads>
		/// Constructor.
		/// </overloads>
		/// <summary>
		/// Constructs a BasicCardService, with default BasicCardSelector.
		/// </summary>
		public BasicCardService()
			: base()
		{
		}

		/// <summary>
		/// Constructs a BasicCardService, with specified card selector.
		/// </summary>
		/// <param name="selector">Card selector to use.</param>
		public BasicCardService(ICardSelector selector)
			: base(selector)
		{
		}

        /// <summary>
        /// Allow using extended length APDU?
        /// </summary>
        public new bool AllowExtendedLength
        {
            get { return base.AllowExtendedLength; }
            set { base.AllowExtendedLength = value; }
        }

        /// <overloads>
        /// Set encryption key.
        /// </overloads>
        /// <param name="keyNumber">number (0..255) or CID of key </param>
        /// <param name="key">encryption key</param>
        public new void SetKey(int keyNumber, byte[] key)
        {
            base.SetKey(keyNumber, key);
        }

        /// <param name="keyComponent">full qualified path of key component</param>
        /// <param name="key">encryption key</param>
        public new void SetKey(string keyComponent, byte[] key)
        {
            base.SetKey(keyComponent, key);
        }

        /// <param name="keyComponent">full qualified path of key component</param>
        /// <param name="key">encryption key</param>
        public new void SetKey(BasicCardString keyComponent, byte[] key)
        {
            base.SetKey(keyComponent.ToString(), key);
        }

		/// <summary>
		/// See <see cref="AbstractBasicCardService.AutoEncryption(int,string)"/>
		/// </summary>
		public new void AutoEncryption(int keyNum, BasicCard.BasicCards.BasicCardString keyName)
		{
			base.AutoEncryption (keyNum, keyName);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.AutoEncryption(int,string)"/>
		/// </summary>
		public new void AutoEncryption(int keyNum, string keyName)
		{
			base.AutoEncryption (keyNum, keyName);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdAssignNad"/>
		/// </summary>
		public new void CmdAssignNad(byte nad)
		{
			base.CmdAssignNad (nad);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdAuthenticateFile"/>
		/// </summary>
		public new void CmdAuthenticateFile(int keyCID, byte[] signature)
		{
			base.CmdAuthenticateFile (keyCID, signature);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdClearEeprom"/>
		/// </summary>
		public new void CmdClearEeprom(int startAddress, int length)
		{
			base.CmdClearEeprom (startAddress, length);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdComponentName"/>
		/// </summary>
		public new BasicCard.BasicCards.BasicCardString CmdComponentName(int CID)
		{
			return base.CmdComponentName (CID);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdCreateComponent"/>
		/// </summary>
		public new int CmdCreateComponent(BasicCard.BasicCards.Components.ComponentType componentType, BasicCard.BasicCards.BasicCardString pathname, byte[] attributes, byte[] data)
		{
			return base.CmdCreateComponent (componentType, pathname, attributes, data);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdDeleteComponent"/>
		/// </summary>
		public new void CmdDeleteComponent(int CID)
		{
			base.CmdDeleteComponent (CID);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdEcho"/>
		/// </summary>
		public new BasicCard.BasicCards.BasicCardString CmdEcho(byte increment, BasicCard.BasicCards.BasicCardString str)
		{
			return base.CmdEcho (increment, str);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdEepromCrc"/>
		/// </summary>
		public new int CmdEepromCrc(int startAddress, int length)
		{
			return base.CmdEepromCrc (startAddress, length);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdEepromSize"/>
		/// </summary>
		public new int[] CmdEepromSize()
		{
			return base.CmdEepromSize ();
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdEndEncryption"/>
		/// </summary>
		public new void CmdEndEncryption()
		{
			base.CmdEndEncryption ();
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdExternalAuthenticate"/>
		/// </summary>
		public new void CmdExternalAuthenticate(int keyCID, CryptAlgo algorithm, byte[] response)
		{
			base.CmdExternalAuthenticate (keyCID, algorithm, response);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdFileIO"/>
		/// </summary>
		public new byte[] CmdFileIO(byte syscode, byte filenum, byte[] commanddata, byte responselen)
		{
			return base.CmdFileIO (syscode, filenum, commanddata, responselen);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdFindComponent"/>
		/// </summary>
		public new int CmdFindComponent(BasicCard.BasicCards.Components.ComponentType componentType, BasicCard.BasicCards.BasicCardString pathname)
		{
			return base.CmdFindComponent (componentType, pathname);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdGetApplicationID()"/>
		/// </summary>
		public new BasicCard.BasicCards.BasicCardString CmdGetApplicationID()
		{
			return base.CmdGetApplicationID ();
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdGetApplicationID()"/>
		/// </summary>
		public new BasicCard.BasicCards.BasicCardString CmdGetApplicationID(int index, int type)
		{
			return base.CmdGetApplicationID (index, type);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdGetChallenge"/>
		/// </summary>
		public new byte[] CmdGetChallenge(int challengeLen)
		{
			return base.CmdGetChallenge (challengeLen);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdGetFreeMemory"/>
		/// </summary>
		public new int[] CmdGetFreeMemory()
		{
			return base.CmdGetFreeMemory ();
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdGetState"/>
		/// </summary>
		public new byte[] CmdGetState()
		{
			return base.CmdGetState ();
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdGrantPrivilege"/>
		/// </summary>
		public new void CmdGrantPrivilege(int privilegeCID, BasicCard.BasicCards.BasicCardString filename)
		{
			base.CmdGrantPrivilege (privilegeCID, filename);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdInternalAuthenticate"/>
		/// </summary>
		public new byte[] CmdInternalAuthenticate(int keyCID, CryptAlgo algorithm, byte[] challenge)
		{
			return base.CmdInternalAuthenticate (keyCID, algorithm, challenge);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdLoadSequence"/>
		/// </summary>
		public new void CmdLoadSequence(LoadSequencePhase phase)
		{
			base.CmdLoadSequence (phase);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdProEncryption"/>
		/// </summary>
		public new void CmdProEncryption(int key, CryptAlgo algorithm)
		{
			base.CmdProEncryption (key, algorithm);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdReadComponentAttr"/>
		/// </summary>
		public new byte[] CmdReadComponentAttr(int CID)
		{
			return base.CmdReadComponentAttr (CID);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdReadComponentData"/>
		/// </summary>
		public new byte[] CmdReadComponentData(int CID)
		{
			return base.CmdReadComponentData (CID);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdReadEeprom"/>
		/// </summary>
		public new byte[] CmdReadEeprom(int startAddress, int length)
		{
			return base.CmdReadEeprom (startAddress, length);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdReadRightsList"/>
		/// </summary>
		public new int[] CmdReadRightsList(int start, int nmax, BasicCard.BasicCards.BasicCardString filename)
		{
			return base.CmdReadRightsList (start, nmax, filename);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdSecureTransportEndSession"/>
		/// </summary>
		public new void CmdSecureTransportEndSession()
		{
			base.CmdSecureTransportEndSession ();
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdSecureTransportStartSession"/>
		/// </summary>
		public new void CmdSecureTransportStartSession(int keyCID, CryptAlgo algorithm, byte[] nonce)
		{
			base.CmdSecureTransportStartSession (keyCID, algorithm, nonce);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdSelectApplication"/>
		/// </summary>
		public new void CmdSelectApplication(BasicCard.BasicCards.BasicCardString filename)
		{
			base.CmdSelectApplication (filename);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdSetState"/>
		/// </summary>
		public new void CmdSetState(byte state)
		{
			base.CmdSetState (state);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdSMAuthentication"/>
		/// </summary>
		public new void CmdSMAuthentication(int keyCID, CryptAlgo algorithm)
		{
			base.CmdSMAuthentication (keyCID, algorithm);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdSMEncryption"/>
		/// </summary>
		public new void CmdSMEncryption(int keyCID, CryptAlgo algorithm, int lenR)
		{
			base.CmdSMEncryption (keyCID, algorithm, lenR);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdStartEncryption"/>
		/// </summary>
		public new void CmdStartEncryption(int key, CryptAlgo algorithm)
		{
			base.CmdStartEncryption (key, algorithm);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdVerify"/>
		/// </summary>
		public new void CmdVerify(int keyCID, BasicCard.BasicCards.BasicCardString password)
		{
			base.CmdVerify (keyCID, password);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdWriteComponentAttr"/>
		/// </summary>
		public new void CmdWriteComponentAttr(int CID, byte[] attributes)
		{
			base.CmdWriteComponentAttr (CID, attributes);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdWriteComponentData"/>
		/// </summary>
		public new void CmdWriteComponentData(int CID, byte[] data)
		{
			base.CmdWriteComponentData (CID, data);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.CmdWriteEeprom"/>
		/// </summary>
		public new void CmdWriteEeprom(int startAddress, byte[] data)
		{
			base.CmdWriteEeprom (startAddress, data);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.DoCommandAndResponse"/>
		/// </summary>
		public new BasicCard.BasicCards.BasicCardResponse DoCommandAndResponse(BasicCard.BasicCards.BasicCardCommand cmd)
		{
			return base.DoCommandAndResponse (cmd);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.ExternalAuthenticate(string,CryptAlgo)"/>
		/// </summary>
		public new void ExternalAuthenticate(BasicCard.BasicCards.BasicCardString keyComponent, CryptAlgo algorithm)
		{
			base.ExternalAuthenticate (keyComponent, algorithm);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.ExternalAuthenticate(string,CryptAlgo)"/>
		/// </summary>
		public new void ExternalAuthenticate(string keyComponent, CryptAlgo algorithm)
		{
			base.ExternalAuthenticate (keyComponent, algorithm);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.FileIO"/>
		/// </summary>
		public new BasicCardFileIO FileIO
		{
			get
			{
				return base.FileIO;
			}
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.InternalAuthenticate(string,CryptAlgo)"/>
		/// </summary>
		public new void InternalAuthenticate(BasicCard.BasicCards.BasicCardString keyComponent, CryptAlgo algorithm)
		{
			base.InternalAuthenticate (keyComponent, algorithm);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.InternalAuthenticate(string,CryptAlgo)"/>
		/// </summary>
		public new void InternalAuthenticate(String keyComponent, CryptAlgo algorithm)
		{
			base.InternalAuthenticate (keyComponent, algorithm);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.ResetCard"/>
		/// </summary>
		public new BasicCard.ProcessorCards.ProcessorCardAtr ResetCard()
		{
			return base.ResetCard ();
		}

		/// <summary>
		/// See <see cref="AbstractCardService.CardSelector"/>
		/// </summary>
		public new ICardSelector CardSelector
		{
			get
			{
				return base.CardSelector;
			}
			set
			{
				base.SetCardSelector(value);
			}
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.SMEncryptionByCid"/>
		/// </summary>
		public new void SMEncryptionByCid(int keyCID, byte[] keyVal, CryptAlgo algorithm)
		{
			base.SMEncryptionByCid (keyCID, keyVal, algorithm);
		}

		/// <summary>
		/// See <see cref="AbstractBasicCardService.SMEncryptionByName(string,byte[],CryptAlgo)"/>
		/// </summary>
		public new void SMEncryptionByName(BasicCard.BasicCards.BasicCardString keyComponent, byte[] keyVal, CryptAlgo algorithm)
		{
			base.SMEncryptionByName (keyComponent, keyVal, algorithm);
		}

		/// <summary>
        /// See <see cref="AbstractBasicCardService.SMEncryptionByName(string,byte[],CryptAlgo)"/>
		/// </summary>
		public new void SMEncryptionByName(string keyComponent, byte[] keyVal, CryptAlgo algorithm)
		{
			base.SMEncryptionByName (keyComponent, keyVal, algorithm);
		}

		/// <summary>
		/// See <see cref="AbstractCardService.Lock"/>
		/// </summary>
		public new void Lock()
		{
			base.Lock ();
		}

		/// <summary>
		/// See <see cref="AbstractCardService.Unlock"/>
		/// </summary>
		public new void Unlock()
		{
			base.Unlock ();
		}

		/// <summary>
		/// See <see cref="ICardService.NeedExclusiveConnection"/>
		/// </summary>
		public new bool NeedExclusiveConnection
		{
			get
			{
				return base.NeedExclusiveConnection;
			}
			set
			{
				doesNeedExclusiveConnection=value;
			}
		}

        /// <summary>
        /// Used connection.
        /// </summary>
        public Terminals.IProcessorCardConnection CardConnection
        {
            get { return pconn; }
        }
	}
}
