/*****************************************************************************/
/* Copyright (c) 2005, ZeitControl cardsystems GmbH                          */
/* All rights reserved.                                                      */
/*                                                                           */
/* Redistribution and use in source and binary forms, with or without        */
/* modification, are generally not permitted. In certain cases the copyright */
/* holder may make an exception to this rule by providing a written licence  */
/* to use, modify and/or redistribute this source code, provided that the    */
/* following conditions are met:                                             */
/*    * Use is limited to options granted by written licence contract.       */
/*    * Redistributions of source code must retain the above copyright       */
/*      notice, this list of conditions and the following disclaimer.        */
/*    * Redistributions in binary form must reproduce the above copyright    */
/*      notice, this list of conditions and the following disclaimer in the  */
/*      documentation and/or other materials provided with the distribution. */
/*    * Neither the name of the company ZeitControl nor the names of its     */
/*      contributors may be used to endorse or promote products derived from */
/*      this software without specific prior written permission.             */
/*                                                                           */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       */
/* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED */
/* TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A           */
/* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER  */
/* OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,  */
/* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,       */
/* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR        */
/* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    */
/* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      */
/* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        */
/* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/*****************************************************************************/
/* Created on 01.04.2005
 * Project: BasicCard .Net Library
 * File description: 
 *   File io exception class.
 * --------------------------------------------------------
 * $Header: /opt/cvsroot/net/BasicCardLib.net/BasicCard/BasicCards/FileSys/BasicCardFileIOException.cs,v 1.2 2011/03/02 17:36:30 michael Exp $
 * $Revision: 1.2 $
 * $Date: 2011/03/02 17:36:30 $
 * --------------------------------------------------------
 * $Log: BasicCardFileIOException.cs,v $
 * Revision 1.2  2011/03/02 17:36:30  michael
 * *** empty log message ***
 *
 * Revision 1.1  2005/07/22 12:57:57  michael
 * Initial checkin
 *
 *---------------------------------------------------------
 */
using System;
using System.IO;
using System.Runtime.Serialization;

namespace BasicCard.BasicCards.FileSys
{
	/// <summary>
	/// BasicCard file error codes.
	/// </summary>
	[CLSCompliant(true), Serializable]
	public enum BasicCardFileError
	{
		/// <summary>
		/// 
		/// </summary>
		feInvalidDrive = 1,
		/// <summary>
		/// 
		/// </summary>
		feBadFilename = 2,
		/// <summary>
		/// 
		/// </summary>
		feBadFilenum = 3,
		/// <summary>
		/// 
		/// </summary>
		feFileNotFound = 4,
		/// <summary>
		/// 
		/// </summary>
		feFileNotOpen = 5,
		/// <summary>
		/// 
		/// </summary>
		feOpenError = 6,
		/// <summary>
		/// 
		/// </summary>
		feSeekError = 7,
		/// <summary>
		/// 
		/// </summary>
		feReadError = 8,
		/// <summary>
		/// 
		/// </summary>
		feWriteError = 9,
		/// <summary>
		/// 
		/// </summary>
		feCloseError = 10,
		/// <summary>
		/// 
		/// </summary>
		feInvalidMode = 11,
		/// <summary>
		/// 
		/// </summary>
		feInvalidAccess = 12,
		/// <summary>
		/// 
		/// </summary>
		feRenameError = 13,
		/// <summary>
		/// 
		/// </summary>
		feAccessDenied = 14,
		/// <summary>
		/// 
		/// </summary>
		feSharingViolation = 15,
		/// <summary>
		/// 
		/// </summary>
		feFileAlreadyExists = 16,
		/// <summary>
		/// 
		/// </summary>
		feNotDataFile = 17,
		/// <summary>
		/// 
		/// </summary>
		feNotDirectory = 18,
		/// <summary>
		/// 
		/// </summary>
		feDirNotEmpty = 19,
		/// <summary>
		/// 
		/// </summary>
		feBadFileChain = 20,
		/// <summary>
		/// 
		/// </summary>
		feFileOpen = 21,
		/// <summary>
		/// 
		/// </summary>
		feNameTooLong = 22,
		/// <summary>
		/// 
		/// </summary>
		feRecordTooLong = 23,
		/// <summary>
		/// 
		/// </summary>
		feTooManyOpenFiles = 24,
		/// <summary>
		/// 
		/// </summary>
		feTooManyCardFiles = 25,
		/// <summary>
		/// 
		/// </summary>
		feCommsError = 26,
		/// <summary>
		/// 
		/// </summary>
		feRemoteFile = 27,
		/// <summary>
		/// 
		/// </summary>
		feNotRemoteFile = 28,
		/// <summary>
		/// 
		/// </summary>
		feRecursiveRename = 29,
		/// <summary>
		/// 
		/// </summary>
		feInvalidFromKeyboard = 30,
		/// <summary>
		/// 
		/// </summary>
		feBadParameter = 31,
		/// <summary>
		/// 
		/// </summary>
		feOutOfMemory = 32,
		/// <summary>
		/// 
		/// </summary>
		feNoFileSystem = 33,
		/// <summary>
		/// 
		/// </summary>
		feUnexpectedError = 34,
		/// <summary>
		/// 
		/// </summary>
		feNotImplemented = 35,
		/// <summary>
		/// 
		/// </summary>
		feTooManyCustomLocks = 36
	}

	/// <summary>
	/// Exceptions caused by BasicCard file io operations.
	/// </summary>
    [CLSCompliant(true), Serializable]
	public class BasicCardFileIOException : IOException, ISerializable
	{
		private BasicCardFileError fe;

		/// <overloads>
		/// Constructor. a BasicCardFileIoException 
		/// </overloads>
		/// <param name="fe">error code</param>
		public BasicCardFileIOException(BasicCardFileError fe)
			: base("Error \"" + ErrorDescription(fe) + "\"")
		{
			this.fe = fe;
		}

		/// <param name="filename">file name</param>
		/// <param name="fe">error code</param>
		public BasicCardFileIOException(string filename, BasicCardFileError fe)
			: base("Error \""	+ ErrorDescription(fe)
			    + "\" in file/directory \"" + filename
			    + "\"")
		{
			this.fe = fe;
		}

        /// <summary>
        /// Serialization constructor
        /// </summary>
        /// <param name="info"></param>
        /// <param name="context"></param>
        public BasicCardFileIOException(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
            fe = (BasicCardFileError)info.GetInt32("BCFIE_fe");            
        }

		/// <summary>
		/// File error code.
		/// </summary>
		public BasicCardFileError ErrorCode
		{
			get
			{
				return fe;
			}
		}

		/// <summary>
		/// Get a describing text for a <see cref="BasicCardFileError"/> code.
		/// </summary>
		/// <param name="fe">error code</param>
		/// <returns>describing text</returns>
		public static string ErrorDescription(BasicCardFileError fe)
		{
			string ret;
			switch (fe)
			{
				case BasicCardFileError.feInvalidDrive :
					ret = "invalid drive";
					break;
				case BasicCardFileError.feBadFilename :
					ret = "bad file name";
					break;
				case BasicCardFileError.feBadFilenum :
					ret = "bad file number";
					break;
				case BasicCardFileError.feFileNotFound :
					ret = "file not found";
					break;
				case BasicCardFileError.feFileNotOpen :
					ret = "file not open";
					break;
				case BasicCardFileError.feOpenError :
					ret = "file open error";
					break;
				case BasicCardFileError.feSeekError :
					ret = "file seek error";
					break;
				case BasicCardFileError.feReadError :
					ret = "file read error";
					break;
				case BasicCardFileError.feWriteError :
					ret = "file write error";
					break;
				case BasicCardFileError.feCloseError :
					ret = "file close error";
					break;
				case BasicCardFileError.feInvalidMode :
					ret = "invalid file mode";
					break;
				case BasicCardFileError.feInvalidAccess :
					ret = "invalid file access";
					break;
				case BasicCardFileError.feRenameError :
					ret = "file/directory rename error";
					break;
				case BasicCardFileError.feAccessDenied :
					ret = "file/directory access denied";
					break;
				case BasicCardFileError.feSharingViolation :
					ret = "file sharing violation";
					break;
				case BasicCardFileError.feFileAlreadyExists :
					ret = "file already exists";
					break;
				case BasicCardFileError.feNotDataFile :
					ret = "not a data file";
					break;
				case BasicCardFileError.feNotDirectory :
					ret = "not a directory";
					break;
				case BasicCardFileError.feDirNotEmpty :
					ret = "directory not empty";
					break;
				case BasicCardFileError.feBadFileChain :
					ret = "bad file chain, file system damaged";
					break;
				case BasicCardFileError.feFileOpen :
					ret = "file open";
					break;
				case BasicCardFileError.feNameTooLong :
					ret = "file/directory name to long";
					break;
				case BasicCardFileError.feRecordTooLong :
					ret = "record to long";
					break;
				case BasicCardFileError.feTooManyOpenFiles :
					ret = "too many open files";
					break;
				case BasicCardFileError.feTooManyCardFiles :
					ret = "too many card files";
					break;
				case BasicCardFileError.feCommsError :
					ret = "communication error";
					break;
				case BasicCardFileError.feRemoteFile :
					ret = "remote file";
					break;
				case BasicCardFileError.feNotRemoteFile :
					ret = "not remote file";
					break;
				case BasicCardFileError.feRecursiveRename :
					ret = "recursive file name";
					break;
				case BasicCardFileError.feInvalidFromKeyboard :
					ret = "invalid from keyboard";
					break;
				case BasicCardFileError.feBadParameter :
					ret = "bad parameter";
					break;
				case BasicCardFileError.feOutOfMemory :
					ret = "out of memory";
					break;
				case BasicCardFileError.feNoFileSystem :
					ret = "no file system";
					break;
				case BasicCardFileError.feUnexpectedError :
				default :
					ret = "unexpected error";
					break;
				case BasicCardFileError.feNotImplemented :
					ret = "not implemented";
					break;
				case BasicCardFileError.feTooManyCustomLocks :
					ret = "too many custom locks";
					break;
			}
			return ret;
		}

        #region ISerializable Members

        /// <inheritdoc/>
        override public void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            base.GetObjectData(info, context);
            info.AddValue("BCFIE_fe", (int)fe);
        }

        #endregion
    }
}
