using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using BasicCard.Framework;

namespace BasicCardExplorer
{
    public partial class MainForm : Form
    {
        private CardReaderEventProvider eventProvider;
        private BasicCardService service=null;

        public MainForm()
        {
            InitializeComponent();
            eventProvider = new CardReaderEventProvider(this);
            eventProvider.RequestService(typeof(BasicCardService), false, null);
            eventProvider.ServiceEvent += new EventHandler<CardServiceEventArgs>(eventProvider_ServiceEvent);
        }

        private void eventProvider_ServiceEvent(object sender, CardServiceEventArgs e)
        {
            switch(e.EventType)
            {
                case CardServiceEventType.ServiceAvailable:
                    if (null == service)
                    {
                        service = (BasicCardService)e.CardService;
                        e.ServiceConsumed = true;
                        UpdateFormData();
                    }
                    break;
                case CardServiceEventType.ServiceUnavailable:
                    service = null;
                    UpdateFormData();
                    break;
            }            
        }

        private void UpdateFormData()
        {
            try
            {
                if (null != service)
                {
                    BasicCardInfo info = service.Info;
                    textBoxAtr.Text = BasicCard.Tools.HexString(service.ATR);
                    textBoxCardReader.Text = service.Reader.Name;
                    textBoxVersion.Text = info.ToString();
                    switch (info.State)
                    {
                        case BasicCardState.New:
                            radioButtonNew.Checked = true;
                            break;
                        case BasicCardState.Load:
                            radioButtonLoad.Checked = true;
                            break;
                        case BasicCardState.Test:
                            radioButtonTest.Checked = true;
                            break;
                        case BasicCardState.Pers:
                            radioButtonPers.Checked = true;
                            break;
                        case BasicCardState.Run:
                            radioButtonRun.Checked = true;
                            break;
                        case BasicCardState.Unknown:
                        default:
                            radioButtonNoBasicCard.Checked = true;
                            break;
                    }
                }
                else
                {
                    textBoxAtr.Text = "";
                    textBoxCardReader.Text = "";
                    textBoxVersion.Text = "";
                    radioButtonNoBasicCard.Checked = true;
                    radioButtonLoad.Enabled = false;
                    radioButtonNew.Enabled = false;
                    radioButtonNoBasicCard.Enabled = false;
                    radioButtonPers.Enabled = false;
                    radioButtonRun.Enabled = false;
                    radioButtonTest.Enabled = false;
                }
            }
            catch (Exception ex)
            {
                textBoxVersion.Text="Exception: "+ex.Message;
            }
        }
    }
}