/*
 * Created on 14.07.2005
 * Project: BasicCard .Net
 * --------------------------------
 * Copyright 2005 ZeitControl cardsystems GmbH
 * --------------------------------
 * $Archive: $
 * $Revision: 1.1 $
 * $Modtime: $
 * --------------------------------
 * $History: $
 */
using System;
using BasicCard;
using BasicCard.Framework;
using BasicCard.GenericCards;

namespace Calc
{
	/// <summary>
	/// Calc example main program.
	/// </summary>
	class MainClass
	{
		private static CalcService calcService;

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			try
			{
				// start the BasicCard framework, so
				// that card readers are monitored
				CardReaderObserver.Instance.Start();
				// ask for a smart card
				System.Console.WriteLine(
						   "Please insert a BasicCard loaded with Calculator example.");
				// request a CalcService object
				// wait for maximum 30 seconds 
				calcService =
					(CalcService) CardReaderObserver
					.Instance
					.WaitForCardService(
						new CalcService(),
						new TimeSpan(0,0,30)); // 0h, 0min, 30s
				if (null == calcService)
				{
					System.Console.WriteLine("No matching BasicCard was inserted.");
					System.Console.WriteLine(
							   "Please load a BasicCard with calc example for using");
					System.Console.WriteLine("this .Net example.");
					System.Console.WriteLine("Press enter to terminate");
					System.Console.ReadLine();
				}
				else
				{
					bool runLoop=true;
					TestExpressions();
					while (runLoop)
					{
						System.Console.WriteLine("Type an expression (H for help),");
						System.Console.WriteLine("and press Enter: ");
						string expression = System.Console.ReadLine();
						expression = expression.Trim();
						switch (expression.ToUpper())
						{
							case "":
							case "Q":
								runLoop=false;
								break;
							case "T":
								TestExpressions();
								break;
							case "H":
								System.Console.WriteLine("Q (or just Enter) to Quit");
								System.Console.WriteLine("T to send test commands");
								break;
							default:
								try
								{
									string result;
									result = calcService.Calculator(expression);
									System.Console.WriteLine(expression + " = " + result);
								}
								catch (CardServiceException e)
								{
									switch ((CalcStatusWord)e.SW1SW2)
									{
										case CalcStatusWord.swSyntaxError :
											System.Console.WriteLine("Syntax error");
											break;
										case CalcStatusWord.swParenthesisMismatch :
											System.Console.WriteLine("Parenthesis mismatch");
											break;
										case CalcStatusWord.swInvalidNumber :
											System.Console.WriteLine("Invalid number");
											break;
										default :
											throw;
									}
								}
								break;
						}
					}
					// Release card reader resource
					calcService.Dispose();
				}
			}
			catch (CommsException e)
			{
				e.Dump();
				System.Console.WriteLine("Press enter to terminate");
				System.Console.ReadLine();
			}
			catch (Exception e)
			{
				System.Console.Error.WriteLine(e.ToString());
				System.Console.WriteLine("Press enter to terminate");
				System.Console.ReadLine();
			}
			finally
			{
				// stop the BasicCard framework
				CardReaderObserver.Instance.Stop();
			}
		}

		private static void TestExpressions()
		{
			System.Console.WriteLine("Decimal test:");
			DecimalTest("1");
			DecimalTest("2 + 2");
			DecimalTest("2+3*4");
			DecimalTest("(2 + 3) * 4");
			DecimalTest("65536 * 32767 + 65535");
			DecimalTest("-65536 * 32768");

			System.Console.WriteLine("Hex test (encrypted):");

			// Start encryption 
			calcService.StartEncryption();

			HexTest("A+3*4");
			HexTest("(c + 3) * 4");
			HexTest("10000 * 7FFF + FFFF");
			HexTest("-10000 * 8000");
			HexTest("-10000 * 4000");

			calcService.EndEncryption();

		}

		private static void DecimalTest(string X)
		{
			System.Console.Write(X + " = ");
			string result = calcService.Calculator(X);
			System.Console.WriteLine(result);
		}

		private static void HexTest(string X)
		{
			System.Console.Write(X + " = ");
			string result = calcService.HexCalculator(X);
			System.Console.WriteLine(result);
		}
	}
}
