/*
 * Created on 14.07.2005
 * Project: BasicCard .Net
 * --------------------------------
 * Copyright 2005 ZeitControl cardsystems GmbH
 * --------------------------------
 * $Archive: $
 * $Revision: 1.1 $
 * $Modtime: $
 * --------------------------------
 * $History: $
 */
using System;
using BasicCard;
using BasicCard.Framework;
using BasicCard.GenericCards;

namespace Debit
{
	/// <summary>
	/// UserExitExceptions are thrown when the program should
	/// be terminated because of user input.
	/// </summary>
	public class UserExitException : Exception
	{
		/// <summary>
		/// Constructs an UserExitException.
		/// </summary>
		public UserExitException()
			: this("Action aborted")
		{
		}

		/// <summary>
		/// Constructs an UserExitException.
		/// </summary>
		/// <param name="message"></param>
		public UserExitException(string message)
			: base(message)
		{
		}
	}

	/// <summary>
	/// Example program for processing debit cards loaded with BasicCard Debit example.
	/// </summary>
	class Dealer
	{
		private static DealerService dealerService;

				

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			CardData finalCarddata=null;
			try
			{
				// start the BasicCard framework, so
				// that card readers are monitored
				CardReaderObserver.Instance.Start();
				// ask for a smart card
				System.Console.WriteLine(
						   "Please insert a BasicCard loaded with Debit example.");
				// request a dealerService object
				// wait for maximum 30 seconds
				dealerService =
					(DealerService) CardReaderObserver
					.Instance
					.WaitForCardService(
					new DealerService(),
					new TimeSpan(0,0,30)); // 0h, 0min, 30s
				if (null == dealerService)
				{
					System.Console.WriteLine("No matching BasicCard was inserted.");
					System.Console.WriteLine(
							   "Please load a BasicCard with debit example for using");
					System.Console.WriteLine("this .Net example.");
				}
				else
				{
					string PIN;
					CardData carddata = dealerService.GetCardData();
					System.Console.WriteLine("Customer: " + carddata.Name);
					System.Console.WriteLine(
							   "Balance: " + carddata.AmountString);

					dealerService.StartEncryption();
					while (true)
					{
						// You probably want to modify the PIN input,
						// which is not secure. Since this example
						// focus on BasicCard not .Net, this is not done in here.
						System.Console.Write("PIN (4 characters, or enter to abort): ");
						PIN = System.Console.ReadLine();
						if (PIN.Length == 0)
							throw new UserExitException();
						if (PIN.Length == 4)
							break; // PIN length
						System.Console.WriteLine("PIN must be 4 characters long");
					}
					dealerService.VerifyPin(PIN);

					System.Console.Write(string.Format("Action (e.g. +50 or -{0}): ", 10.25));
					decimal Amount = Decimal.Parse(System.Console.ReadLine());
					if (Amount >= 0)
					{
						dealerService.IncreaseAmount(Amount);
					}
					else
					{
						dealerService.DecreaseAmount(-Amount);
					}
					finalCarddata=dealerService.GetCardData();
					dealerService.Dispose(); // release smart card resources
					System.Console.WriteLine("Action succesful");
				}
			}
			catch (CommsException ex)
			{
				if (null != dealerService)
				{
					dealerService.ShowMessage(ex);
				}
				else
				{
					ex.Dump();
				}
			}
			catch (UserExitException ex)
			{
				System.Console.Error.WriteLine(ex.Message);
				// terminate
			}
			catch (Exception ex)
			{
				System.Console.Error.WriteLine(ex.ToString());
			}
			finally
			{
				// stop the BasicCard framework
				CardReaderObserver.Instance.Stop();
			}
			if (null!=finalCarddata)
			{
				System.Console.WriteLine(
					"New Balance: " + finalCarddata.AmountString);
			}
			System.Console.WriteLine("Press enter to terminate");
			System.Console.ReadLine();
		}
	}
}
