/*
 * Created on 14.07.2005
 * Project: BasicCard .Net
 * --------------------------------
 * Copyright 2005 ZeitControl cardsystems GmbH
 * --------------------------------
 * $Archive: $
 * $Revision: 1.1 $
 * $Modtime: $
 * --------------------------------
 * $History: $
 */
using System;
using BasicCard;
using BasicCard.Framework;
using BasicCard.GenericCards;


namespace Debit
{
	/// <summary>
	/// UserExitExceptions are thrown when the program should
	/// be terminated because of user input.
	/// </summary>
	public class UserExitException : Exception
	{
		/// <summary>
		/// Constructs an UserExitException.
		/// </summary>
		public UserExitException()
			: this("Personalization aborted")
		{
		}

		/// <summary>
		/// Constructs an UserExitException.
		/// </summary>
		/// <param name="message"></param>
		public UserExitException(string message)
			: base(message)
		{
		}
	}

	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	class Issuer
	{
		private static IssuerService issuerService;

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			try
			{
				// start the BasicCard framework, so
				// that card readers are monitored
				CardReaderObserver.Instance.Start();
				// ask for a smart card
				System.Console.WriteLine(
						   "Please insert a BasicCard loaded with Debit example.");
				// request a dealerService object
				// wait for maximum 30 seconds
				issuerService =
					(IssuerService) CardReaderObserver
					.Instance
					.WaitForCardService(
					new IssuerService(),
					new TimeSpan(0,0,30)); // 0h, 0min, 30s
				if (null == issuerService)
				{
					System.Console.WriteLine("No matching BasicCard was inserted.");
					System.Console.WriteLine(
							   "Please load a BasicCard with debit example for using");
					System.Console.WriteLine("this .Net example.");
				}
				else
				{
					decimal Amount;
					string PIN;
					string Name;
					System.Console.Write("Customer name (Enter to abort): ");
					Name = System.Console.ReadLine();
					if (Name.Length == 0)
						throw new UserExitException();

					while (true)
					{
						System.Console.Write("PIN (4 characters, or enter to abort): ");
						PIN = System.Console.ReadLine();
						if (PIN.Length == 0)
							throw new UserExitException();
						if (PIN.Length == 4)
							break; // PIN length
						System.Console.WriteLine("PIN must be 4 characters long");
					}

					while (true)
					{
						System.Console.Write(string.Format("Initial balance (e.g. {0:0.00}): ", 12.50));
						try
						{
							Amount = decimal.Parse(System.Console.ReadLine());
                            break;
						}
						catch (FormatException)
						{
							System.Console.WriteLine("Invalid number");
						}
					}

					issuerService.StartEncryption();
					issuerService.PersonaliseCard(Amount, PIN, Name);
					issuerService.Dispose(); // release smart card resources
					System.Console.WriteLine("Personalization succesful");
				}
			}
			catch (CommsException ex)
			{
				if (null != issuerService)
				{
					issuerService.ShowMessage(ex);
				}
				else
				{
					ex.Dump();
				}
			}
			catch (UserExitException ex)
			{
				System.Console.Error.WriteLine(ex.Message);
				// terminate
			}
			catch (Exception ex)
			{
				System.Console.Error.WriteLine(ex.ToString());
			}
			finally
			{
				// stop the BasicCard framework
				CardReaderObserver.Instance.Stop();
			}
			System.Console.WriteLine("Press enter to terminate");
			System.Console.ReadLine();
		}
	}
}
