/*
 * Created on 14.07.2005
 * Project: BasicCard .Net
 * --------------------------------
 * Copyright 2005 ZeitControl cardsystems GmbH
 * --------------------------------
 * $Archive: $
 * $Revision: 1.1 $
 * $Modtime: $
 * --------------------------------
 * $History: $
 */
using System;
using BasicCard;
using BasicCard.BasicCards;

namespace Debit
{
	/// <summary>
	/// Service class for managing issuer functions for BasicCard debit example card.
	/// </summary>
	public class IssuerService : DealerService
	{
		/// <summary>
		/// Constructs a IssuerService object
		/// </summary>
		public IssuerService()
			: base()
		{
			byte[] key =
			{
				(byte) 0xF1,
				(byte) 0xA8,
				(byte) 0xF4,
				(byte) 0x3E,
				(byte) 0x04,
				(byte) 0x04,
				(byte) 0x8D,
				(byte) 0xEB };
			SetKey(0, key);
		}


		/// <summary>
		/// Override of start encryption using issuer key.
		/// </summary>
		public override void StartEncryption()
		{
			AutoEncryption(0, "\\Key0");
		}

		/// <summary>
		/// Call PersonaliseCard command.
		/// </summary>
		/// <param name="Amount">new amount of card</param>
		/// <param name="NewPIN">new PIN for this card</param>
		/// <param name="Name">card owner name</param>
		public void PersonaliseCard(decimal Amount, string NewPIN, string Name)
		{
			BasicCardCommand cmd = new BasicCardCommand(0x80, 0x00);
			cmd.AppendBasicLong(FromDecimal(Amount));
			cmd.AppendBasicStringN(
			new BasicCardStringN(4, NewPIN, BasicCardString.CharsetIdAscii));
			cmd.AppendBasicString(new BasicCardString(Name, StringEncoding));
			cmd.DisableLE(); // we do not need response data
			BasicCardResponse rsp = DoCommandAndResponse(cmd);
			rsp.CheckSW1SW2();
		}
	}
}
