' Created on 14.07.2005
' Project: BasicCard .Net
' --------------------------------
' Copyright 2005 ZeitControl cardsystems GmbH
' --------------------------------
' $Archive: $
' $Revision: 1.1 $
' $Modtime: $
' --------------------------------
' $History: $
Option Explicit On 
Imports BasicCard
Imports BasicCard.BasicCards

Public Enum CalcStatusWord
    swSyntaxError = &H6481
    swParenthesisMismatch = &H6482
    swInvalidNumber = &H6483
End Enum

Public Class CalcService
    Inherits AbstractBasicCardService

    Public Sub New()
        ' specify Application ID of card we like to use
        MyBase.New("BasicCard Mini-Calculator")
        doesNeedExclusiveConnection = False ' we like to share the card
        ' set the encryption key see Calckeys.bas
        Dim key() As Byte = {&HD1, &H47, &H52, &HF8, _
        &HE2, &HD8, &HFE, &HDA}
        SetKey(1, key)
    End Sub


    ' Activate encryption.
    Public Sub StartEncryption()
        ' If we are using the card none exclusive,
        ' others may access the same card at the same time.
        ' This will not work if encryption is active. So
        ' in this case lock the card first for exclusive use.
        If Not IsExclusive Then Lock()
        ' Activate encryption by use of autoEncryption function.
        ' Depending on BasicCard type either key 1 (single application
        ' BasicCard) or key component "\Key1" (MultiApplication BasicCard)
        ' is used within card. To use autoEncryption the specified key
        ' (1) must be known by the base class AbstractBasicCardService.
        ' For this it must have been set by use of setKey function (see
        ' CalcService constructor).
        AutoEncryption(1, "\Key1")
    End Sub

    ' Deactivate encryption.
    Public Sub EndEncryption()
        ' End active encryption
        CmdEndEncryption()
        ' Release exclusive use
        Unlock()
    End Sub


    ' Call calculator command by calling one of following BasicCard commands:
    ' Declare Command &H20 &H01 Calculator (Expression$)
    ' Declare Command &H20 &H01 HexCalculator (P1=1, Expression$)
    ' hexadecimal: if true expression must be hexadecimal, otherwise decimal
    ' expression: expression to evaluate
    ' returns result
    Protected Function IntCalc(ByVal hexadecimal As Boolean, ByVal expression As String) As String
        ' Construct a new BasicCardCommand object with CLA=0x20 (&H20)
        ' and INS=&H01
        Dim cmd As BasicCardCommand = New BasicCardCommand(&H20, &H1)
        If hexadecimal Then
            ' we want to call HexCalculator, set P1 to 1
            cmd.P1 = 1
        End If
        ' BasicCard strings and .Net strings vary regarding encoding.
        ' A BasicCard string uses 8 bit per character, a .Net string uses
        ' 16 bit per character (Unicode). Such we use our BasicCardString
        ' class when sending strings to BasicCard. The BasicCardString class
        ' provides translations between .Net and BasicCard strings. Please 
        ' note: It depends on your application and expected string contents
        ' which translation is a proper choice. Here we expect to use ACSII
        ' characters only, such CharsetIdAscii is used.
        Dim bsExpression As BasicCardString = _
          New BasicCardString(expression, BasicCardString.CharsetIdAscii)
        ' Add expression string as first parameter to the BasicCardCommand object
        cmd.AppendBasicString(bsExpression)
        ' Call the BasicCard command, now the command is transmitted to the BasicCard. 
        Dim rsp As BasicCardResponse = DoCommandAndResponse(cmd)
        ' We have received the response from BasicCard. Check SW1SW2 for
        ' error conditions. In case SW1SW2 is neither 0x9000 nor 0x61xx this
        ' will cause a CommsExecption to be thrown.
        rsp.CheckSW1SW2()
        ' Get the expression string returned from card. Again we get a BasicCardString
        ' object, not a Java String object.
        bsExpression = rsp.FetchBasicString()
        ' Translate the response expression to a valid Java string by use of
        ' ASCII character set.
        Return bsExpression.ToString(BasicCardString.CharsetIdAscii)
    End Function

    ' Call calculator command by calling the following BasicCard command:
    ' Declare Command &H20 &H01 Calculator (Expression$)
    ' expression: Expression in decimal format to evaluate, e.g. "10*9".
    ' returns Resulting expression string, e.g. "90".
    Public Function Calculator(ByVal expression As String) As String
        Return IntCalc(False, expression)
    End Function

    ' Call calculator command by calling the following BasicCard command:
    ' Declare Command &H20 &H01 HexCalculator (P1=1, Expression$)
    ' expression: Expression in hexadecimal format to evaluate, e.g. "1F+A".
    ' returns Resulting expression string, e.g. "29".
    Public Function HexCalculator(ByVal expression As String) As String
        Return IntCalc(True, expression)
    End Function
End Class
