' Created on 14.07.2005
' Project: BasicCard .Net
' --------------------------------
' Copyright 2005 ZeitControl cardsystems GmbH
' --------------------------------
' $Archive: $
' $Revision: 1.1 $
' $Modtime: $
' --------------------------------
' $History: $
Option Explicit On 
Imports BasicCard
Imports BasicCard.Framework
Imports BasicCard.GenericCards

Module MainModule

    Private calcService As calcService

    Sub Main()
        Try
            ' start the BasicCard framework, so
            ' that card readers are monitored
            CardReaderObserver.Instance.Start()
            ' ask for a smart card
            System.Console.WriteLine( _
                  "Please insert a BasicCard loaded with Calculator example.")
            ' request a CalcService object
            ' wait for maximum 30 seconds 
            calcService = _
                     CardReaderObserver.Instance().WaitForCardService( _
               New CalcService, _
               New TimeSpan(0, 0, 30)) ' 0h, 0min, 30s
            If calcService Is Nothing Then
                System.Console.WriteLine("No matching BasicCard was inserted.")
                System.Console.WriteLine("Please load a BasicCard with calc example for using")
                System.Console.WriteLine("this .Net example.")
                System.Console.WriteLine("Press enter to terminate")
                System.Console.ReadLine()
            Else
                Dim runLoop As Boolean = True
                TestExpressions()
                While (runLoop)
                    System.Console.WriteLine("Type an expression (H for help),")
                    System.Console.WriteLine("and press Enter: ")
                    Dim expression As String = System.Console.ReadLine()
                    expression = expression.Trim()
                    Select Case expression.ToUpper()
                        Case "", "Q"
                            runLoop = False
                        Case "T"
                            TestExpressions()
                        Case "H"
                            System.Console.WriteLine("Q (or just Enter) to Quit")
                            System.Console.WriteLine("T to send test commands")
                        Case Else
                            Try
                                Dim result As String
                                result = calcService.Calculator(expression)
                                System.Console.WriteLine(expression + " = " + result)
                            Catch e As CardServiceException
                                Select Case e.SW1SW2
                                    Case CalcStatusWord.swSyntaxError
                                        System.Console.WriteLine("Syntax error")
                                    Case CalcStatusWord.swParenthesisMismatch
                                        System.Console.WriteLine("Parenthesis mismatch")
                                    Case CalcStatusWord.swInvalidNumber
                                        System.Console.WriteLine("Invalid number")
                                    Case Else
                                        Throw
                                End Select
                            End Try
                    End Select
                End While
                ' Release card reader resource
                calcService.Dispose()
            End If
        Catch e As CommsException
            e.Dump()
            System.Console.WriteLine("Press enter to terminate")
            System.Console.ReadLine()
        Catch e As Exception
            System.Console.Error.WriteLine(e.ToString())
            System.Console.WriteLine("Press enter to terminate")
            System.Console.ReadLine()
        Finally
            ' stop the BasicCard framework
            CardReaderObserver.Instance.Stop()
        End Try
    End Sub

    Private Sub TestExpressions()
        System.Console.WriteLine("Decimal test:")
        DecimalTest("1")
        DecimalTest("2 + 2")
        DecimalTest("2+3*4")
        DecimalTest("(2 + 3) * 4")
        DecimalTest("65536 * 32767 + 65535")
        DecimalTest("-65536 * 32768")

        System.Console.WriteLine("Hex test (encrypted):")

        ' Start encryption 
        calcService.StartEncryption()

        HexTest("A+3*4")
        HexTest("(c + 3) * 4")
        HexTest("10000 * 7FFF + FFFF")
        HexTest("-10000 * 8000")
        HexTest("-10000 * 4000")

        calcService.EndEncryption()
    End Sub


    Private Sub DecimalTest(ByVal X As String)
        System.Console.Write(X + " = ")
        Dim result As String = calcService.Calculator(X)
        System.Console.WriteLine(result)
    End Sub

    Private Sub HexTest(ByVal X As String)
        System.Console.Write(X + " = ")
        Dim result As String = calcService.HexCalculator(X)
        System.Console.WriteLine(result)
    End Sub

End Module
