' Created on 14.07.2005
' Project: BasicCard .Net
' --------------------------------
' Copyright 2005 ZeitControl cardsystems GmbH
' --------------------------------
' $Archive: $
' $Revision: 1.2 $
' $Modtime: $
' --------------------------------
' $History: $
Option Explicit On 
Imports BasicCard
Imports BasicCard.Framework
Imports BasicCard.GenericCards

' UserExitExceptions are thrown when the program should
' be terminated because of user input.
Public Class UserExitException
    Inherits Exception

    ' Constructs an UserExitException.
    Public Sub New()
        MyBase.New("Action aborted")
    End Sub

		
    ' Constructs an UserExitException.
    Public Sub New(ByVal message As String)
        MyBase.New(message)
    End Sub
End Class

Module MainModule

    Private dealerService As DealerService

    Sub Main()
        Dim finalCarddata As CardData = Nothing
        Try
            ' start the BasicCard framework, so
            ' that card readers are monitored
            CardReaderObserver.Instance.Start()
            ' ask for a smart card
            System.Console.WriteLine( _
                  "Please insert a BasicCard loaded with Debit example.")
            ' request a DealerService object
            ' wait for maximum 30 seconds 
            dealerService = _
                     CardReaderObserver.Instance().WaitForCardService( _
               New DealerService, _
               New TimeSpan(0, 0, 30)) ' 0h, 0min, 30s
            If dealerService Is Nothing Then
                System.Console.WriteLine("No matching BasicCard was inserted.")
                System.Console.WriteLine("Please load a BasicCard with debit example for using")
                System.Console.WriteLine("this .Net example.")
                System.Console.WriteLine("Press enter to terminate")
                System.Console.ReadLine()
            Else
                Dim PIN As String = Nothing
                Dim carddata As CardData = dealerService.GetCardData()
                System.Console.WriteLine("Customer: " + carddata.Name)
                System.Console.WriteLine("Balance: " + carddata.AmountString)
                dealerService.StartEncryption()
                While (True)
                    ' You probably want to modify the PIN input,
                    ' which is not secure. Since this example
                    ' focus on BasicCard not .Net, this is not done in here.
                    System.Console.Write("PIN (4 characters, or enter to abort): ")
                    PIN = System.Console.ReadLine()
                    If 0 = PIN.Length Then
                        Throw New UserExitException
                    End If
                    If 4 = PIN.Length Then
                        Exit While
                    End If
                    System.Console.WriteLine("PIN must be 4 characters long")
                End While
                dealerService.VerifyPin(PIN)

                System.Console.Write(String.Format("Action (e.g. +50 or -{0}): ", 10.25))
                Dim Amount As Decimal = Decimal.Parse(System.Console.ReadLine())
                If Amount >= 0 Then
                    dealerService.IncreaseAmount(Amount)
                Else
                    dealerService.DecreaseAmount(-Amount)
                End If
                finalCarddata = dealerService.GetCardData()
                dealerService.Dispose() ' release smart card resources
                System.Console.WriteLine("Action succesful")
            End If
        Catch e As CommsException
            e.Dump()
            System.Console.WriteLine("Press enter to terminate")
            System.Console.ReadLine()
        Catch e As Exception
            System.Console.Error.WriteLine(e.ToString())
            System.Console.WriteLine("Press enter to terminate")
            System.Console.ReadLine()
        Finally
            ' stop the BasicCard framework
            CardReaderObserver.Instance.Stop()
        End Try
        If Not (finalCarddata Is Nothing) Then
            System.Console.WriteLine("New Balance: " + finalCarddata.AmountString)
        End If
        System.Console.WriteLine("Press enter to terminate")
        System.Console.ReadLine()
    End Sub

End Module
