' Created on 14.07.2005
' Project: BasicCard .Net
' --------------------------------
' Copyright 2005 ZeitControl cardsystems GmbH
' --------------------------------
' $Archive: $
' $Revision: 1.2 $
' $Modtime: $
' --------------------------------
' $History: $

Option Explicit On 
Imports BasicCard
Imports BasicCard.Framework
Imports BasicCard.GenericCards


' UserExitExceptions are thrown when the program should
' be terminated because of user input.
Public Class UserExitException
    Inherits Exception

    ' Constructs an UserExitException.
    Public Sub New()
        MyBase.New("Personalization aborted")
    End Sub


    ' Constructs an UserExitException.
    Public Sub New(ByVal message As String)
        MyBase.New(message)
    End Sub
End Class

Module MainModule

    Private issuerService As IssuerService

    Sub Main()
        Try
            ' start the BasicCard framework, so
            ' that card readers are monitored
            CardReaderObserver.Instance.Start()
            ' ask for a smart card
            System.Console.WriteLine( _
                  "Please insert a BasicCard loaded with Debit example.")
            ' request a DealerService object
            ' wait for maximum 30 seconds 
            issuerService = _
                     CardReaderObserver.Instance().WaitForCardService( _
               New IssuerService, _
               New TimeSpan(0, 0, 30)) ' 0h, 0min, 30s
            If issuerService Is Nothing Then
                System.Console.WriteLine("No matching BasicCard was inserted.")
                System.Console.WriteLine("Please load a BasicCard with debit example for using")
                System.Console.WriteLine("this .Net example.")
                System.Console.WriteLine("Press enter to terminate")
                System.Console.ReadLine()
            Else
                Dim Amount As Decimal
                Dim PIN As String = Nothing
                Dim Name As String
                System.Console.Write("Customer name (Enter to abort): ")
                Name = System.Console.ReadLine()
                If Name.Length = 0 Then
                    Throw New UserExitException
                End If

                While (True)
                    System.Console.Write("PIN (4 characters, or enter to abort): ")
                    PIN = System.Console.ReadLine()
                    If PIN.Length = 0 Then
                        Throw New UserExitException
                    End If
                    If PIN.Length = 4 Then
                        Exit While ' PIN length
                    End If
                    System.Console.WriteLine("PIN must be 4 characters long")
                End While

                While (True)
                    System.Console.Write(String.Format("Initial balance (e.g. {0:0.00}): ", 12.5))
                    Try
                        Amount = Decimal.Parse(System.Console.ReadLine())
                        Exit While
                    Catch e As FormatException
                        System.Console.WriteLine("Invalid number")
                    End Try
                End While
                issuerService.StartEncryption()
                issuerService.PersonaliseCard(Amount, PIN, Name)
                issuerService.Dispose() ' release smart card resources
                System.Console.WriteLine("Personalization succesful")
            End If
        Catch e As CommsException
            e.Dump()
            System.Console.WriteLine("Press enter to terminate")
            System.Console.ReadLine()
        Catch e As Exception
            System.Console.Error.WriteLine(e.ToString())
            System.Console.WriteLine("Press enter to terminate")
            System.Console.ReadLine()
        Finally
            ' stop the BasicCard framework
            CardReaderObserver.Instance.Stop()
        End Try
        System.Console.WriteLine("Press enter to terminate")
        System.Console.ReadLine()
    End Sub

End Module
